import numpy as np
from gym.envs.mujoco import mujoco_env
from gym import utils
from RockClimbEnv import RockWallGenerator
import os


def mass_center(model, sim):
    mass = np.expand_dims(model.body_mass, 1)
    xpos = sim.data.xipos
    return (np.sum(mass * xpos, 0) / np.sum(mass))[0]


class HumanRockClimbEnv(mujoco_env.MujocoEnv, utils.EzPickle):
    def __init__(self):
        src_path = os.path.join(os.path.dirname(__file__), 'assets', 'humanoid.xml')
        random_wall = RockWallGenerator.insert_wall(src_path)
        target_path = os.path.join(os.path.dirname(__file__), 'assets', 'human_with_wall.xml')
        with open(target_path, 'wb') as outfile:
            random_wall.write(outfile)
        print(target_path)
        mujoco_env.MujocoEnv.__init__(self, target_path, 5)
        utils.EzPickle.__init__(self)

    def _get_obs(self):
        data = self.sim.data
        return np.concatenate([data.qpos.flat[2:],
                               data.qvel.flat,
                               data.cinert.flat,
                               data.cvel.flat,
                               data.qfrc_actuator.flat,
                               data.cfrc_ext.flat])

    def step(self, a):
        """

        Parameters
        ----------
        action :

        Returns
        -------
        ob, reward, episode_over, info : tuple
            ob (object) :
                an environment-specific object representing your observation of
                the environment.
            reward (float) :
                amount of reward achieved by the previous action. The scale
                varies between environments, but the goal is always to increase
                your total reward.
            episode_over (bool) :
                whether it's time to reset the environment again. Most (but not
                all) tasks are divided up into well-defined episodes, and done
                being True indicates the episode has terminated. (For example,
                perhaps the pole tipped too far, or you lost your last life.)
            info (dict) :
                 diagnostic information useful for debugging. It can sometimes
                 be useful for learning (for example, it might contain the raw
                 probabilities behind the environment's last state change).
                 However, official evaluations of your agent are not allowed to
                 use this for learning.
        """
        pos_before = mass_center(self.model, self.sim)
        self.do_simulation(a, self.frame_skip)
        pos_after = mass_center(self.model, self.sim)
        alive_bonus = 5.0
        data = self.sim.data
        lin_vel_cost = 0.25 * (pos_after - pos_before) / self.model.opt.timestep
        quad_ctrl_cost = 0.1 * np.square(data.ctrl).sum()
        quad_impact_cost = .5e-6 * np.square(data.cfrc_ext).sum()
        quad_impact_cost = min(quad_impact_cost, 10)
        reward = lin_vel_cost - quad_ctrl_cost - quad_impact_cost + alive_bonus
        qpos = self.sim.data.qpos
        done = bool(qpos[1] > 1.0)  # bool((qpos[2] < -1.0) or (qpos[2] > 9.0))
        return self._get_obs(), reward, done, dict(reward_linvel=lin_vel_cost, reward_quadctrl=-quad_ctrl_cost,
                                                   reward_alive=alive_bonus, reward_impact=-quad_impact_cost)

    def reset_model(self):
        c = 0
        temp_q_pos = self.init_qpos + self.np_random.uniform(low=-c, high=c, size=self.model.nq)
        temp_q_pos[1] -= 2
        self.set_state(
            temp_q_pos,
            self.init_qvel + self.np_random.uniform(low=-c, high=c, size=self.model.nv, )
        )
        return self._get_obs()

    def viewer_setup(self):
        pass
        # self.viewer.cam.trackbodyid = 1
        # self.viewer.cam.distance = self.model.stat.extent * 1.0
        # self.viewer.cam.lookat[2] += .8
        # self.viewer.cam.elevation = -20














        # class HumanRockClimbEnv(gym.Env):
        #     metadata = {'render.modes': ['human']}

        #     def __init__(self):
        #         pass

        #     def _step(self, action):

        #         self._take_action(action)
        #         self.status = self.env.step()
        #         reward = self._get_reward()
        #         ob = self.env.getState()
        #         episode_over = self.status != hfo_py.IN_GAME
        #         return ob, reward, episode_over, {}

        #     def _reset(self):
        #         pass

        #     def _render(self, mode='human', close=False):
        #         pass

        #     def _take_action(self, action):
        #         pass

        #     def _get_reward(self):
        #         """ Reward is given for XY. """
        #         if self.status == FOOBAR:
        #             return 1
        #         elif self.status == ABC:
        #             return self.somestate ** 2
        #         else:
        #             return 0
